diff -durN kamailio-3.2.3/forward.c kamailio-3.2.3-new/forward.c
--- kamailio-3.2.3/forward.c	2012-04-19 15:10:24.000000000 +0300
+++ kamailio-3.2.3-new/forward.c	2012-05-17 13:30:09.000000000 +0300
@@ -754,6 +754,7 @@
 /* removes first via & sends msg to the second */
 int forward_reply(struct sip_msg* msg)
 {
+	struct ip_addr ip; /* debugging only */
 	char* new_buf;
 	struct dest_info dst;
 	unsigned int new_len;
@@ -777,9 +778,15 @@
 		}
 	}
 	
+	new_buf = build_res_buf_from_sip_res( msg, &new_len);
+	if (!new_buf){
+		LOG(L_ERR, "ERROR: forward_reply: building failed\n");
+		goto error;
+	}
+
 	/* check modules response_f functions */
 	for (r=0; r<mod_response_cbk_no; r++)
-		if (mod_response_cbks[r](msg)==0) goto skip;
+	    mod_response_cbks[r](msg);
 	/* we have to forward the reply stateless, so we need second via -bogdan*/
 	if (parse_headers( msg, HDR_VIA2_F, 0 )==-1 
 		|| (msg->via2==0) || (msg->via2->error!=PARSE_OK))
@@ -789,11 +796,6 @@
 		goto error;
 	}
 
-	new_buf = build_res_buf_from_sip_res( msg, &new_len);
-	if (!new_buf){
-		LOG(L_ERR, "ERROR: forward_reply: building failed\n");
-		goto error;
-	}
 
 	dst.proto=msg->via2->proto;
 	SND_FLAGS_OR(&dst.send_flags, &msg->fwd_send_flags, &msg->rpl_send_flags);
@@ -831,7 +833,8 @@
 				
 	} 
 #endif
-
+	
+	
 	apply_force_send_socket(&dst, msg);
 
 	if (msg_send(&dst, new_buf, new_len)<0)
@@ -839,6 +842,26 @@
 		STATS_RPL_FWD_DROP();
 		goto error;
 	}
+
+	/* call onsend_route */
+	if(dst.send_sock == NULL) {
+		dst.send_sock=get_send_socket(msg, &dst.to, dst.proto);
+		if (dst.send_sock==0){
+			LOG(L_ERR, "forward_reply: ERROR: cannot forward reply\n");
+			goto error;
+		}
+	}
+	
+	if (run_onsend(msg, &dst, new_buf, new_len)==0)
+	{	
+	    su2ip_addr(&ip, &dst.to);
+	    LOG(L_INFO, "forward_reply: reply to %s:%d(%d) dropped"
+	                " (onsend_route)\n", ip_addr2a(&ip),
+	                su_getport(&dst.to), dst.proto);
+		
+	    goto error;
+	}
+
 #ifdef STATS
 	STATS_TX_RESPONSE(  (msg->first_line.u.reply.statuscode/100) );
 #endif
@@ -849,7 +872,7 @@
 
 	STATS_RPL_FWD_OK();
 	pkg_free(new_buf);
-skip:
+
 	return 0;
 error:
 	if (new_buf) pkg_free(new_buf);
